const int QUALITY_FRESHFISH = 1;	//the quality of freshly-caught fish (for selling to Gordon)

const int BASE_CATCH_CHANCE = 20;	//20% base chance of catching a fish, if any are about
const int CHANCE_PER_FISH	= 5;	//5% extra catch chance per nearby fish

const int SOUND_LANDEDFISH	= 40;	//played when you catch a fish

//Reconstituted from original function, with some extra bits added (more advice to the player, tweaked catch chance)
FishingRod 662 ()
{
	var target;
	var avatar_pos;
	var fish_caught;
	var num_fish_nearby;
	var nearby_fish;
	var fish;
	var caught_fish;
	var rand;

	if (event != DOUBLECLICK) return;

	UI_close_gumps();

	target = UI_click_on_item();

	UI_execute_usecode_array(AVATAR, [0x67, 0x61]);	//fishing animation?

	if (!(UI_is_water([target[2], target[3], target[4]]) || UI_get_item_shape(target) == SHAPE_FISH))
	{
		UI_flash_mouse(0);

		//only make this 1 in 4, to stop irritating the player
		if (UI_get_random(4) == 1)
		{
			//Spark is the resident angling expert
			if (NPCInParty(SPARK)) SPARK.say("\"Thou shouldst cast thy line into the water, Avatar!\" He grins cheekily.");
			else randomPartySay("\"Last I heard, fish doth not live upon dry land. Thou shouldst try thy rod in the water!\"");
		}
		return;
	}

	nearby_fish = UI_find_nearby(AVATAR, SHAPE_FISH, 15, 0);
	num_fish_nearby = UI_get_array_size(nearby_fish);

	//Adjusted fish-catching chance: now it is a base chance + an adjustable chance per fish
	if (num_fish_nearby > 0)
	{
		var catch_chance = BASE_CATCH_CHANCE + (num_fish_nearby * CHANCE_PER_FISH);
		if (UI_get_random(100) <= catch_chance) fish_caught = true;
	}

	if (fish_caught)
	{
		fish = UI_create_new_object(SHAPE_FOOD);
		if (!fish) return;

		UI_set_item_frame(fish, FRAME_TROUT);

		UI_set_item_flag(fish, OKAY_TO_TAKE);
		UI_set_item_flag(fish, TEMPORARY);		//Added to stop fish sticking around


		//put it near the avatar's feet
		avatar_pos = UI_get_object_position(AVATAR);
		avatar_pos[X] = avatar_pos[X] + 1;
		UI_update_last_created(avatar_pos);

		UI_play_sound_effect2(SOUND_LANDEDFISH, fish);

		//Added: indicates that the fish is fresh (for sale to Gordon)
		//Note: this required a new tfa.dat, to change the class of the Food shape from Quality Flags to Quality.
		UI_set_item_quality(fish, QUALITY_FRESHFISH);


		//Added: remove a fish from the water
		if (UI_get_item_shape(target) == SHAPE_FISH) caught_fish = target;	//Player clicked on a particular fish - grab that one
		else caught_fish = nearby_fish[UI_get_random(num_fish_nearby)];		//otherwise, grab one of the nearby fishies at random
		UI_remove_item(caught_fish);

		//Make one of the party members comment on the catch
		rand = UI_get_random(3);
		if (rand == 1)
		{
			randomPartyBark("\"Indeed, a whopper!\"");
			if (UI_npc_nearby(SPARK)) delayedBark(SPARK, "\"I have seen bigger.\"", 16);
		}
		else if (rand == 2) randomPartyBark("\"What a meal!\"");
		else if (rand == 3)	randomPartyBark(["\"That fish does not", "look right.\""]);
	}

	//changed barks to reflect that there's no fish (better player guidance)
	else if (num_fish_nearby == 0)
	{
		rand = UI_get_random(3);
		if (rand == 1)
		{
			//Spark, butting his head in as usual
			if (NPCInParty(SPARK))
			{
				SPARK.say("\"The fish doth not seem to be biting, thou shouldst try somewhere else!~\"My father took me fishing sometimes -- thou canst find good spots near bridges, where thou canst see the fishes beneath the water.\"");
			}
			else randomPartySay("\"Methinks there are no fish in these waters. Let us try in a better spot!\"");
		}
		else if (rand == 2) delayedBark(AVATAR, "\"Not even a bite!\"", 0);
		else if (rand == 3) delayedBark(AVATAR, "\"I've lost my bait.\"", 0);
	}

	else
	{
		rand = UI_get_random(4);

		if (rand == 1) delayedBark(AVATAR, "\"Not even a bite!\"", 0);
		else if (rand == 2)
		{
			delayedBark(AVATAR, "\"It got away!\"", 1);
			if (UI_npc_nearby(IOLO)) delayedBark(IOLO, "\"It was the Big One!\"", 16);
		}
		else if (rand == 3) delayedBark(AVATAR, "\"I've lost my bait.\"", 0);
		else if (rand == 4) delayedBark(AVATAR, "\"I felt a nibble.\"", 0);
	}
}