/* This script contains edible food behaviours */

enum food_levels
{
	FOODLEVEL_PECKISH	= 5,	//below this level, the character is starving (taking damage from hunger?)
	FOODLEVEL_WELLFED	= 10,	//below this level, the character is peckish (just complaining about hunger?)
	FOODLEVEL_STUFFED	= 20,	//at or above this level, the character is stuffed (this changes their barks)
	FOODLEVEL_FULL		= 25	//at or above this level, the character is full (refuses more food)
};

//This had to be overridden in order to separate the choosing of the target (now handled by UseEdible) from the act of eating the food
//Note that it has a different order of arguments from useEdible(), in order to make it remotely sensible
consumeEdible 0xC03 (var food, var npc, var nutritional_value, var sound_effect)
{
	var npc_food_level;			//The current hunger level of the character
	var level_after_feeding;	//What hunger level the character will be at if fed this item
	var bark;					//What the character will say when fed
	var rand;					//Random number, used for randomising responses

	//Todo: Work out which flags these actually are, so they can be given meaningful constants
	if (!NPCInParty(npc) ||
		UI_get_item_flag(npc, 1) ||
		UI_get_item_flag(npc, 4) ||
		UI_get_item_flag(npc, 7)
	) return;


	//Added in custom behavior for some NPCs, when faced with the daunting prospect of milk products
	//In future, lactose-intolerance should be handled by a custom flag
	var npcnum			= UI_get_npc_number(npc);
	var shapenum		= UI_get_item_shape(food);
	var framenum		= UI_get_item_frame(food);
	var cheesy_frames	= [FRAME_BUTTER, FRAME_CHEESE1, FRAME_CHEESE2, FRAME_CHEESE3];

	//Poor Iolo is lactose intolerant.
	if (npc == UI_get_npc_object(IOLO) &&
		((shapenum == SHAPE_FOOD && framenum in cheesy_frames) ||
		(shapenum == SHAPE_BOTTLE && framenum == FRAME_MILK)))
	{
		UI_item_say(npc, "\"Nay, I am lactose-intolerant.\"");
		return;
	}

	//Jaana's just health-conscious. Tch, these healers.
	//Tweaked 2005-03-17: Jaana has decided that cheese is ok, so she only objects to butter now
	else if (npc == UI_get_npc_object(JAANA) &&
		shapenum == SHAPE_FOOD && framenum == FRAME_BUTTER)
	{
		npc->say("\"No thank thee, I am watching my cholesterol. As shouldst thou!\""); //sanctimonious cow
		return;
	}

	npc_food_level = UI_get_npc_prop(npc, FOODLEVEL);
	level_after_feeding = npc_food_level + nutritional_value;

	//Character is full already, do not pass go
	if (npc_food_level >= FOODLEVEL_FULL)
	{
		UI_item_say(npc, "\"No, thank thee.\"");
		return;
	}

	checkIfItemStolen(food);
	UI_play_sound_effect2(sound_effect, npc);	//Homph homph homph
	subtractItemQuantity(food);

	rand = UI_get_random(10);	//Used for randomising the barks


	//measure the NPC's current food level and their resulting food level to determine barks

	//character was stuffed to start with
	if (npc_food_level >= FOODLEVEL_STUFFED)
	{
		if (framenum == FRAME_BUTTER && rand < 6) bark = "\"Urgh, my arteries...\"";

		//no idea why it's checking a global flag - was this a debug flag to test barks or something?
		else if (rand > 4 || (gflags[0x009B] && rand < 2)) bark = "\"I'll soon be plump.\"";
	}

	//character wasn't terribly hungry to start with
	else if (npc_food_level >= FOODLEVEL_PECKISH)
	{
		//chowed down on garlic
		if (shapenum == SHAPE_REAGENT) bark = "\"Yum, garlic!\"";

		//fed with something really filling
		else if (level_after_feeding >= FOODLEVEL_FULL && rand < 3) bark = "\"Belch\"";

		else bark = "\"Ahh, very tasty.\"";	
	}

	//for the following cases, the character was hungry to start with

	//character is now pretty full
	else if (level_after_feeding >= FOODLEVEL_STUFFED)
	{
		if (rand > 5) bark = "\"That hit the spot!\"";
		else bark = "\"Burp\"";	
	}

	//character is now happily fed
	else if (level_after_feeding >= FOODLEVEL_WELLFED)
	{
		if (shapenum == SHAPE_REAGENT) bark = "\"Yum, garlic!\"";
		else bark = "\"Ah yes, much better.\"";		
	}

	//character is still peckish
	else if (level_after_feeding >= FOODLEVEL_PECKISH)
	{
		if (rand > 5 || isAvatar(npc)) bark = "\"I am still hungry.\"";
		else bark = "\"May I have some more?\"";	
	}

	//character is still starving
	else
	{
		if (rand > 5) bark = "\"More!\"";
		else bark = "\"I must have more!\"";
	}

	if (bark != "" && NPCCanTalk(npc)) UI_item_say(npc, bark);

	UI_set_npc_prop(npc, FOODLEVEL, level_after_feeding);
	return;
}

//Called when food is clicked on - picks target and then feeds the food to the target
//This has been rewritten so that the actual eating is handled by another function - this just controls the click behavior
useEdible 0x813 (var sound_effect, var nutritional_value, var food)
{
	var npc = UI_click_on_item();
	consumeEdible(food, npc, nutritional_value, sound_effect);
	return;
}

//Called when user clicks on a bottle
//(This has been rewired so that we can use milk bottles on butter churns: see churn.uc for more details)

const int SOUND_DRINK = 90;
const int BOTTLE_NUTRITION = 1;

extern gotoChurn 0xB13(var churn, var milktype);	//defined in churn.uc
Bottle 616 ()
{
	//Bottle was clicked on
	if (event == DOUBLECLICK)
	{
		//this is a milk bottle - go and pick it up before you use it (necessary for arcane reasons)
		if (UI_get_item_frame(item) == FRAME_MILK)
		{
			//avatar is already carrying the milk - run Bottle() again with SCRIPTED this time
			if (containedByAvatar(item))
			{
				event = SCRIPTED;
				Bottle();
				return;
			}
			else gotoAndGet(item, 0xB01, item, SCRIPTED);
		}
		else
		{
			useEdible(SOUND_DRINK, BOTTLE_NUTRITION, item);
			return;
		}
	}

	//Milk was picked up, and is ready to be used
	else if (event == SCRIPTED)
	{
		var target = UI_click_on_item();
		//a churn was selected - go to the churn and run churnButter()
		if (UI_get_item_shape(target) == SHAPE_KITCHEN_ITEM && UI_get_item_frame(target) == FRAME_CHURN) gotoChurn(target, 1);

		//otherwise, just drink the damn milk already
		else consumeEdible(item, target, BOTTLE_NUTRITION, SOUND_DRINK);	
	}
}

//Pick milk up off the ground and calls gotoChurn()
getMilk 0xB01()	{ pickUpItem(item, SHAPE_BOTTLE, item, SCRIPTED); }